// udpqueue.cpp
//
// A delay queue for UDP packets.
//
//   (C) Copyright 2011 Fred Gleason <fredg@paravelsystems.com>
//
//    $Id: udpqueue.cpp,v 1.1 2011/02/28 23:36:42 cvs Exp $
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License version 2 as
//   published by the Free Software Foundation.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public
//   License along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <syslog.h>
#include <sys/time.h>

#include <qsocketnotifier.h>

#include "udpqueue.h"

UdpQueue::UdpQueue(Profile *p,int id,QObject *parent,const char *name)
  : QObject(parent,name)
{
  queue_id=id;
  queue_delay=0.0;
  queue_fixed_delay=0.0;
  queue_destructive_dump=true;

  //
  // UDP Socket
  //
  queue_socket=new QSocketDevice(QSocketDevice::Datagram);
  queue_socket->setBlocking(false);
  if(!Load(p)) {
    return;
  }
  if(!queue_socket->bind(QHostAddress(),queue_input_port)) {
    syslog(LOG_ERR,"unable to bind input socket for [UdpQueue%d]",id);
    return;
  }
  QSocketNotifier *notify=new QSocketNotifier(queue_socket->socket(),
					      QSocketNotifier::Read,this);
  connect(notify,SIGNAL(activated(int)),this,SLOT(readyReadData(int)));

  //
  // Queue Timer
  //
  queue_udp_timer=new QTimer(this);
  connect(queue_udp_timer,SIGNAL(timeout()),this,SLOT(checkQueueData()));
}


UdpQueue::~UdpQueue()
{
}


void UdpQueue::changeDelayState(int id,Cunctator::DelayState state,int len)
{
  queue_delay=(double)len/1000.0;
}


void UdpQueue::delayDumped(int id)
{
  if(queue_destructive_dump) {
    while(queue_udp_datas.size()>0) {
      queue_udp_datas.pop();
      queue_udp_lengths.pop();
      queue_udp_timestamps.pop();
    }
  }    
}


void UdpQueue::readyReadData(int fd)
{
  int n;
  char data[1500];

  while((n=queue_socket->readBlock(data,1500))>0) {
    queue_udp_datas.push(new char[n]);
    memcpy(queue_udp_datas.back(),data,n);
    queue_udp_lengths.push(n);
    queue_udp_timestamps.push(GetTimestamp());
  } 
  if(!queue_udp_timer->isActive()) {
    queue_udp_timer->start(CUNC_UDP_QUEUE_POLL_INTERVAL);
  }
}


void UdpQueue::checkQueueData()
{
  double timestamp=GetTimestamp();

  while((queue_udp_timestamps.size()>0)&&
	((queue_udp_timestamps.front()+queue_delay+queue_fixed_delay)<
	 timestamp)) {
    for(unsigned i=0;i<queue_destination_addresses.size();i++) {
      queue_socket->writeBlock(queue_udp_datas.front(),
			       queue_udp_lengths.front(),
			       queue_destination_addresses[i],
			       queue_destination_ports[i]);
    }
    delete queue_udp_datas.front();
    queue_udp_datas.pop();
    queue_udp_lengths.pop();
    queue_udp_timestamps.pop();
  }

  if(queue_udp_datas.size()==0) {
    queue_udp_timer->stop();
  }
}


bool UdpQueue::Load(Profile *p)
{
  QString section=QString().sprintf("UdpQueue%d",queue_id+1);
  QHostAddress addr;
  int port;
  int delay;
  int count=0;

  if((delay_id=p->intValue(section,"DelayNumber",-1))<0) {
    syslog(LOG_ERR,"missing/invalid DelayNumber= value in %s\n",
	   (const char *)section);
    return false;
  }
  if((queue_input_port=p->intValue(section,"InputPort",-1))<0) {
    syslog(LOG_ERR,"missing/invalid InputPort= value in %s\n",
	   (const char *)section);
    return false;
  }
  queue_fixed_delay=(double)p->intValue(section,"FixedDelay",0)/1000;
  queue_destructive_dump=p->boolValue(section,"DestructiveDump",true);

  addr.setAddress(p->stringValue(section,
	       QString().sprintf("DestinationAddress%d",count+1),"0.0.0.0"));
  while(!addr.isNull()) {
    port=p->intValue(section,QString().sprintf("DestinationPort%d",count+1),0);
    if((port>0)&&(port<0x10000)) {
      queue_destination_addresses.push_back(addr);
      queue_destination_ports.push_back(port);
      queue_fixed_delays.push_back(delay);
    }
    else {
      syslog(LOG_ERR,"missing/invalid DestinationPort%d= value in %s\n",
	     count+1,(const char *)section);
    }
    count++;
    addr.setAddress(p->stringValue(section,
		QString().sprintf("DestinationAddress%d",count+1),"0.0.0.0"));
  }

  return true;
}


double UdpQueue::GetTimestamp()
{
  struct timeval tv;

  gettimeofday(&tv,NULL);
  return (double)tv.tv_sec+(double)tv.tv_usec/1000000.0;
}
